@@ -27,7 +27,7 @@ class DocumentationGenerator(object):
         api_docs = []
         for api in apis:
             api_docs.append({
-                'description': IntrospectorHelper.get_view_description(api['callback']),
+                'description': IntrospectorHelper.get_summary(api['callback']),
                 'path': api['path'],
                 'operations': self.get_operations(api, apis),
             })
@@ -222,6 +222,10 @@ class DocumentationGenerator(object):
                 serializer = self._get_method_serializer(method_introspector)
                 if serializer is not None:
                     serializers.add(serializer)
+                extras = method_introspector.get_extra_serializer_classes()
+                for extra in extras:
+                    if extra is not None:
+                        serializers.add(extra)
 
         return serializers
 
