@@ -7,7 +7,7 @@ import re
 import yaml
 import importlib
 
-from .compat import OrderedDict
+from .compat import OrderedDict, strip_tags
 from abc import ABCMeta, abstractmethod
 
 from django.http import HttpRequest
@@ -16,13 +16,23 @@ from django.contrib.admindocs.utils import trim_docstring
 from django.utils.encoding import smart_text
 
 import rest_framework
-from rest_framework.views import get_view_name, get_view_description
+from rest_framework.views import get_view_name
 from rest_framework import viewsets
 from rest_framework.compat import apply_markdown
 from rest_framework.utils import formatting
 from django.utils import six
 
 
+def get_view_description(view_cls, html=False, docstring=None):
+    if docstring is not None:
+        view_cls = type(
+            view_cls.__name__ + '_fake',
+            (view_cls,),
+            {'__doc__': docstring})
+    return rest_framework.settings.api_settings \
+        .VIEW_DESCRIPTION_FUNCTION(view_cls, html)
+
+
 def get_default_value(field):
     default_value = getattr(field, 'default', None)
     if rest_framework.VERSION >= '3.0.0':
@@ -45,9 +55,10 @@ class IntrospectorHelper(object):
         split_lines = trim_docstring(docstring).split('\n')
 
         cut_off = None
-        for index, line in enumerate(split_lines):
+        for index in range(len(split_lines) - 1, -1, -1):
+            line = split_lines[index]
             line = line.strip()
-            if line.startswith('---'):
+            if line == '---':
                 cut_off = index
                 break
         if cut_off is not None:
@@ -61,12 +72,13 @@ class IntrospectorHelper(object):
         Strips the params from the docstring (ie. myparam -- Some param) will
         not be removed from the text body
         """
+        params_pattern = re.compile(r'(?:^|[^-])--(?:$|[^-])')
         split_lines = trim_docstring(docstring).split('\n')
 
         cut_off = None
         for index, line in enumerate(split_lines):
             line = line.strip()
-            if line.find('--') != -1:
+            if params_pattern.search(line):
                 cut_off = index
                 break
         if cut_off is not None:
@@ -90,14 +102,13 @@ class IntrospectorHelper(object):
         return serializer.__class__.__name__
 
     @staticmethod
-    def get_view_description(callback):
+    def get_summary(callback, docstring=None):
         """
         Returns the first sentence of the first line of the class docstring
         """
-        from .compat import strip_tags
-        description = get_view_description(callback).split("\n")[0].split(".")[0]
-        if apply_markdown:
-            description = strip_tags(do_markdown(description))
+        description = strip_tags(get_view_description(
+            callback, html=True, docstring=docstring)) \
+            .split("\n")[0].split(".")[0]
         return description
 
 
@@ -124,10 +135,10 @@ class BaseViewIntrospector(object):
         """
         Returns the first sentence of the first line of the class docstring
         """
-        return IntrospectorHelper.get_view_description(self.callback)
+        return IntrospectorHelper.get_summary(self.callback)
 
     def get_docs(self):
-        return self.callback.__doc__
+        return get_view_description(self.callback)
 
 
 class BaseMethodIntrospector(object):
@@ -169,6 +180,10 @@ class BaseMethodIntrospector(object):
         parser.object = new_object
         return parser
 
+    def get_extra_serializer_classes(self):
+        return self.get_yaml_parser().get_extra_serializer_classes(
+            self.callback)
+
     def ask_for_serializer_class(self):
         if hasattr(self.callback, 'get_serializer_class'):
             view = self.create_view()
@@ -211,17 +226,10 @@ class BaseMethodIntrospector(object):
         return serializer
 
     def get_summary(self):
-        docs = self.get_docs()
-
         # If there is no docstring on the method, get class docs
-        if docs is None:
-            docs = self.parent.get_description()
-        docs = trim_docstring(docs).split("\n")[0].split(".")[0]
-        if apply_markdown:
-            from .compat import strip_tags
-            docs = strip_tags(do_markdown(docs))
-
-        return docs
+        return IntrospectorHelper.get_summary(
+            self.callback,
+            self.get_docs() or self.parent.get_description())
 
     def get_nickname(self):
         """ Returns the APIView's nickname """
@@ -235,8 +243,7 @@ class BaseMethodIntrospector(object):
         """
         docstring = ""
 
-        class_docs = self.callback.__doc__ or ''
-        class_docs = smart_text(class_docs)
+        class_docs = get_view_description(self.callback)
         class_docs = IntrospectorHelper.strip_yaml_from_docstring(class_docs)
         class_docs = IntrospectorHelper.strip_params_from_docstring(class_docs)
         method_docs = self.get_docs()
@@ -265,7 +272,7 @@ class BaseMethodIntrospector(object):
         path_params = self.build_path_parameters()
         body_params = self.build_body_parameters()
         form_params = self.build_form_parameters()
-        query_params = self.build_query_params_from_docstring()
+        query_params = self.build_query_parameters()
 
         if path_params:
             params += path_params
@@ -296,7 +303,8 @@ class BaseMethodIntrospector(object):
         method = str(self.method).lower()
         if not hasattr(self.callback, method):
             return None
-        return getattr(self.callback, method).__doc__
+
+        return get_view_description(getattr(self.callback, method))
 
     def build_body_parameters(self):
         serializer = self.get_request_serializer_class()
@@ -328,7 +336,7 @@ class BaseMethodIntrospector(object):
 
         return params
 
-    def build_query_params_from_docstring(self):
+    def build_query_parameters(self):
         params = []
 
         docstring = self.retrieve_docstring() or ''
@@ -410,8 +418,6 @@ def get_data_type(field):
         return field.type_label
     elif isinstance(field, fields.BooleanField):
         return 'boolean'
-    elif isinstance(field, fields.CharField):
-        return 'string'
     elif isinstance(field, fields.URLField):
         return 'url'
     elif isinstance(field, fields.SlugField):
@@ -434,10 +440,12 @@ def get_data_type(field):
         return 'float'
     elif isinstance(field, fields.DecimalField):
         return 'decimal'
-    elif isinstance(field, fields.FileField):
-        return 'file'
     elif isinstance(field, fields.ImageField):
         return 'image upload'
+    elif isinstance(field, fields.FileField):
+        return 'file upload'
+    elif isinstance(field, fields.CharField):
+        return 'string'
     else:
         return 'field'
 
@@ -460,11 +468,13 @@ class WrappedAPIViewIntrospector(BaseViewIntrospector):
         return self.callback().allowed_methods
 
     def get_notes(self):
-        class_docs = self.callback.__doc__ or ''
-        class_docs = smart_text(class_docs)
-        class_docs = IntrospectorHelper.strip_yaml_from_docstring(class_docs)
-        class_docs = IntrospectorHelper.strip_params_from_docstring(class_docs)
-        return do_markdown(class_docs)
+        class_docs = get_view_description(self.callback)
+        class_docs = IntrospectorHelper.strip_yaml_from_docstring(
+            class_docs)
+        class_docs = IntrospectorHelper.strip_params_from_docstring(
+            class_docs)
+        return get_view_description(
+            self.callback, html=True, docstring=class_docs)
 
 
 def do_markdown(docstring):
@@ -492,7 +502,7 @@ class WrappedAPIViewMethodIntrospector(BaseMethodIntrospector):
         endpoint. If none are available, the class docstring
         will be used
         """
-        return self.callback.__doc__
+        return get_view_description(self.callback)
 
     def get_module(self):
         from rest_framework_swagger.decorators import wrapper_to_func
@@ -544,14 +554,17 @@ class ViewSetIntrospector(BaseViewIntrospector):
 
             freevars = x.code.co_freevars
         except (AttributeError, IndexError):
-            raise RuntimeError('Unable to use callback invalid closure/function specified.')
+            raise RuntimeError(
+                'Unable to use callback invalid closure/function ' +
+                'specified.')
         else:
             return x.closure[freevars.index('actions')].cell_contents
 
 
 class ViewSetMethodIntrospector(BaseMethodIntrospector):
     def __init__(self, view_introspector, method, http_method):
-        super(ViewSetMethodIntrospector, self).__init__(view_introspector, method)
+        super(ViewSetMethodIntrospector, self) \
+            .__init__(view_introspector, method)
         self.http_method = http_method.upper()
 
     def get_http_method(self):
@@ -572,6 +585,23 @@ class ViewSetMethodIntrospector(BaseMethodIntrospector):
         view.request.method = self.http_method
         return view
 
+    def build_query_parameters(self):
+        parameters = super(ViewSetMethodIntrospector, self) \
+            .build_query_parameters()
+        view = self.create_view()
+        if self.method == 'list' and view.paginate_by:
+            parameters.append({'paramType': 'query',
+                               'name': view.page_kwarg,
+                               'description': None,
+                               'dataType': 'integer'})
+        if self.method == 'list' and view.paginate_by and \
+                view.paginate_by_param:
+            parameters.append({'paramType': 'query',
+                               'name': view.paginate_by_param,
+                               'description': None,
+                               'dataType': 'integer'})
+        return parameters
+
 
 def multi_getattr(obj, attr, default=None):
     """
@@ -823,6 +853,22 @@ class YAMLDocstringParser(object):
             pass
         return None
 
+    def get_extra_serializer_classes(self, callback):
+        """
+        Retrieves serializer classes from pytype YAML objects
+        """
+        parameters = self.object.get('parameters', [])
+        serializers = []
+        for parameter in parameters:
+            serializer = parameter.get('pytype', None)
+            if serializer is not None:
+                try:
+                    serializer = self._load_class(serializer, callback)
+                    serializers.append(serializer)
+                except (ImportError, ValueError):
+                    pass
+        return serializers
+
     def get_request_serializer_class(self, callback):
         """
         Retrieves request serializer class from YAML object
@@ -873,7 +919,7 @@ class YAMLDocstringParser(object):
             view_mocker = self._load_class(view_mocker, callback)
         return view_mocker
 
-    def get_parameters(self):
+    def get_parameters(self, callback):
         """
         Retrieves parameters from YAML object
         """
@@ -889,6 +935,14 @@ class YAMLDocstringParser(object):
             # https://github.com/wordnik/swagger-core/wiki/1.2-transition#wiki-additions-2
             # https://github.com/wordnik/swagger-core/wiki/Parameters
             data_type = field.get('type', 'string')
+            pytype = field.get('pytype', None)
+            if pytype is not None:
+                try:
+                    serializer = self._load_class(pytype, callback)
+                    data_type = IntrospectorHelper.get_serializer_name(
+                        serializer)
+                except (ImportError, ValueError):
+                    pass
             if param_type in ['path', 'query', 'header']:
                 if data_type not in BaseMethodIntrospector.PRIMITIVES:
                     data_type = 'string'
@@ -948,7 +1002,7 @@ class YAMLDocstringParser(object):
         from method and docstring
         """
         parameters = []
-        docstring_params = self.get_parameters()
+        docstring_params = self.get_parameters(inspector.callback)
         method_params = inspector.get_parameters()
 
         # paramType may differ, overwrite first
