@@ -6,6 +6,7 @@ from django.conf.urls import patterns, url, include
 from django.contrib.auth.models import User
 from django.contrib.admindocs.utils import trim_docstring
 from django.test import TestCase
+from django.test.utils import override_settings
 from django.utils.importlib import import_module
 from django.views.generic import View
 
@@ -21,8 +22,8 @@ from .decorators import wrapper_to_func, func_to_wrapper
 from .urlparser import UrlParser
 from .docgenerator import DocumentationGenerator
 from .introspectors import ViewSetIntrospector, APIViewIntrospector, \
-    WrappedAPIViewIntrospector, WrappedAPIViewMethodIntrospector, \
-    IntrospectorHelper, APIViewMethodIntrospector
+    WrappedAPIViewMethodIntrospector, IntrospectorHelper, \
+    APIViewMethodIntrospector
 
 
 def no_markdown(func):
@@ -587,22 +588,26 @@ def get_introspectors(introspector):
     return dict((x.method, x) for x in iter(introspector))
 
 
+def make_viewset_introspector(view_class):
+    return ViewSetIntrospector(
+        view_class,
+        '/api/endpoint/{pk}',
+        url(
+            r'^/api/endpoint/(?P<{pk}>[^/]+)$',
+            view_class.as_view({
+                'get': 'list',
+                'post': 'create',
+                'put': 'update',
+                'patch': 'partial_update',
+                'delete': 'destroy'
+            })
+        )
+    )
+
+
 class ViewSetMethodIntrospectorTests(TestCase):
     def make_view_introspector(self, view_class):
-        return ViewSetIntrospector(
-            view_class,
-            '/api/endpoint/{pk}',
-            url(
-                r'^/api/endpoint/(?P<{pk}>[^/]+)$',
-                view_class.as_view({
-                    'get': 'retrieve',
-                    'post': 'create',
-                    'put': 'update',
-                    'patch': 'partial_update',
-                    'delete': 'destroy'
-                })
-            )
-        )
+        return make_viewset_introspector(view_class)
 
     def test_get_serializer_class_access_action(self):
         class MyViewSet(ModelViewSet):
@@ -616,9 +621,11 @@ class ViewSetMethodIntrospectorTests(TestCase):
 
         introspector = self.make_view_introspector(MyViewSet)
         method_introspectors = get_introspectors(introspector)
-        serializer_class = method_introspectors['create'].get_serializer_class()
+        serializer_class = method_introspectors['create'] \
+            .get_serializer_class()
         self.assertIs(serializer_class, CommentSerializer)
-        serializer_class = method_introspectors['retrieve'].get_serializer_class()
+        serializer_class = method_introspectors['update'] \
+            .get_serializer_class()
         self.assertIs(serializer_class, QuerySerializer)
 
     def test_get_serializer_class_access_method(self):
@@ -630,10 +637,124 @@ class ViewSetMethodIntrospectorTests(TestCase):
                     return CommentSerializer
                 return QuerySerializer
         class_introspector = self.make_view_introspector(MyViewSet)
-        introspector = get_introspectors(class_introspector)['retrieve']
-        self.assertEqual(CommentSerializer, introspector.get_serializer_class())
+        introspector = get_introspectors(class_introspector)['list']
+        self.assertEqual(
+            CommentSerializer,
+            introspector.get_serializer_class())
         introspector = get_introspectors(class_introspector)['create']
-        self.assertEqual(QuerySerializer, introspector.get_serializer_class())
+        self.assertEqual(
+            QuerySerializer,
+            introspector.get_serializer_class())
+
+    def test_builds_pagination_parameters_list(self):
+        class MyViewSet(ModelViewSet):
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['list']
+        params = introspector.build_query_parameters()
+        page = [p for p in params if p['name'] == 'page']
+        self.assertEqual(1, len(page))
+        page_by = [p for p in params if p['name'] == 'page_this_by']
+        self.assertEqual(1, len(page_by))
+
+    def test_no_builds_pagination_parameters_for_create(self):
+        class MyViewSet(ModelViewSet):
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        params = introspector.build_query_parameters()
+        page = [p for p in params if p['name'] == 'page']
+        self.assertEqual(0, len(page))
+        page_by = [p for p in params if p['name'] == 'page_this_by']
+        self.assertEqual(0, len(page_by))
+
+    def test_get_summary_empty(self):
+        class MyViewSet(ModelViewSet):
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("", summary)
+
+    def test_get_summary_view(self):
+        class MyViewSet(ModelViewSet):
+            """
+            *Slimy angels*
+            """
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("Slimy angels", summary)
+
+    @no_markdown
+    def test_get_summary_view_nomarkdown(self):
+        class MyViewSet(ModelViewSet):
+            """
+            *Slimy angels*
+            """
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("*Slimy angels*", summary)
+
+    def test_get_summary_method(self):
+        class MyViewSet(ModelViewSet):
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+            def create(self, request):
+                """
+                *Slimy angels*
+                """
+                pass
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("Slimy angels", summary)
+
+    @no_markdown
+    def test_get_summary_method_nomarkdown(self):
+        class MyViewSet(ModelViewSet):
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+            def create(self, request):
+                """
+                *Slimy angels*
+                """
+                pass
+
+        class_introspector = self.make_view_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("*Slimy angels*", summary)
 
 
 class BaseViewIntrospectorTest(TestCase):
@@ -798,6 +919,32 @@ class BaseMethodIntrospectorTest(TestCase):
         self.assertEqual(params[0]['name'], 'email')
         self.assertIsNone(params[0]['defaultValue'])
 
+        url_patterns = patterns('', url(r'my-api/', SerializedAPI.as_view()))
+        urlparser = UrlParser()
+        generator = DocumentationGenerator()
+        apis = urlparser.get_apis(url_patterns)
+        models = generator.get_models(apis)
+        self.assertIn("SomeSerializer", models)
+        properties = models["SomeSerializer"]['properties']
+        self.assertEqual("email", properties["email"]["type"])
+        self.assertEqual("string", properties["content"]["type"])
+        self.assertEqual("datetime", properties["created"]["type"])
+        self.assertEqual("date", properties["expires"]["type"])
+        self.assertEqual("time", properties["expires_by"]["type"])
+        self.assertEqual("integer", properties["age"]["type"])
+        self.assertEqual("boolean", properties["flagged"]["type"])
+        self.assertEqual("url", properties["url"]["type"])
+        self.assertEqual("slug", properties["slug"]["type"])
+        self.assertIn(
+            properties["choice"]["type"],
+            ["choice", "multiple choice"])
+        self.assertEqual("regex", properties["regex"]["type"])
+        self.assertEqual("float", properties["float"]["type"])
+        self.assertEqual("decimal", properties["decimal"]["type"])
+        self.assertEqual("file upload", properties["file"]["type"])
+        self.assertEqual("image upload", properties["image"]["type"])
+        self.assertEqual("field", properties["joop"]["type"])
+
     def test_build_form_parameters_allowable_values(self):
 
         class MySerializer(serializers.Serializer):
@@ -878,6 +1025,12 @@ class YAMLDocstringParserTests(TestCase):
         return APIViewIntrospector(
             view_class, '/', RegexURLResolver(r'^/$', ''))
 
+    def make_fbv_introspector(self, view):
+        from .introspectors import WrappedAPIViewIntrospector
+        return WrappedAPIViewIntrospector(
+            func_to_wrapper(view), '/', RegexURLResolver(r'^/$', '')
+        )
+
     def test_yaml_loader(self):
         class AnAPIView(APIView):
             def get(self):
@@ -1195,9 +1348,7 @@ class YAMLDocstringParserTests(TestCase):
             """
             return "blarg"
 
-        class_introspector = WrappedAPIViewIntrospector(
-            func_to_wrapper(SerializedAPI2), '/', RegexURLResolver(r'^/$', '')
-        )
+        class_introspector = self.make_fbv_introspector(SerializedAPI2)
         introspector = WrappedAPIViewMethodIntrospector(class_introspector, 'POST')
         generator = DocumentationGenerator()
         serializer = generator._get_method_serializer(introspector)
@@ -1213,9 +1364,7 @@ class YAMLDocstringParserTests(TestCase):
             """
             return "blarg"
 
-        class_introspector = WrappedAPIViewIntrospector(
-            SerializedAPI2.cls, '/', RegexURLResolver(r'^/$', '')
-        )
+        class_introspector = self.make_fbv_introspector(SerializedAPI2)
         wrapper_to_func(SerializedAPI2.cls)
         introspector = WrappedAPIViewMethodIntrospector(class_introspector, 'POST')
         generator = DocumentationGenerator()
@@ -1232,9 +1381,7 @@ class YAMLDocstringParserTests(TestCase):
             """
             return "blarg"
 
-        class_introspector = WrappedAPIViewIntrospector(
-            SerializedAPI2.cls, '/', RegexURLResolver(r'^/$', '')
-        )
+        class_introspector = self.make_fbv_introspector(SerializedAPI2)
         wrapper_to_func(SerializedAPI2.cls)
         introspector = WrappedAPIViewMethodIntrospector(class_introspector, 'POST')
         generator = DocumentationGenerator()
@@ -1251,10 +1398,7 @@ class YAMLDocstringParserTests(TestCase):
             """
             return "blarg"
 
-        class_introspector = WrappedAPIViewIntrospector(
-            SerializedAPI2.cls, '/', RegexURLResolver(r'^/$', '')
-        )
-        wrapper_to_func(SerializedAPI2.cls)
+        class_introspector = self.make_fbv_introspector(SerializedAPI2)
         introspector = WrappedAPIViewMethodIntrospector(class_introspector, 'POST')
         generator = DocumentationGenerator()
         try:
@@ -1583,10 +1727,7 @@ class YAMLDocstringParserTests(TestCase):
             """
             return "blarg"
 
-        class_introspector = WrappedAPIViewIntrospector(
-            func_to_wrapper(a_view), '/', RegexURLResolver(r'^/$', '')
-        )
-
+        class_introspector = self.make_fbv_introspector(a_view)
         notes = class_introspector.get_notes()
         self.assertEqual(notes, "Slimy toads")
         introspector = WrappedAPIViewMethodIntrospector(class_introspector, 'POST')
@@ -1604,10 +1745,7 @@ class YAMLDocstringParserTests(TestCase):
             """
             return "blarg"
 
-        class_introspector = WrappedAPIViewIntrospector(
-            func_to_wrapper(a_view), '/', RegexURLResolver(r'^/$', '')
-        )
-
+        class_introspector = self.make_fbv_introspector(a_view)
         url_patterns = patterns('', url(r'my-api/', a_view))
         urlparser = UrlParser()
         generator = DocumentationGenerator()
@@ -1662,7 +1800,6 @@ class YAMLDocstringParserTests(TestCase):
         self.assertEqual(CommentSerializer, serializer)
 
     def test_view_mocker_null(self):
-
         class SerializedAPI(ListCreateAPIView):
             def get_serializer_class(self):
                 if self.request.tacos == 'tasty':
@@ -1682,6 +1819,40 @@ class YAMLDocstringParserTests(TestCase):
         serializer = generator._get_method_serializer(introspector)
         self.assertIsNone(serializer)
 
+    def test_pytype(self):
+        @api_view(["POST"])
+        def a_view(request):
+            """
+            ---
+            response_serializer: CommentSerializer
+            parameters:
+                - name: stuff
+                  paramType: body
+                  pytype: QuerySerializer
+                  type: int
+            """
+            from rest_framework.views import Response
+            return Response("o noes!")
+        generator = DocumentationGenerator()
+        urlparser = UrlParser()
+        url_patterns = patterns('', url(r'my-api/', a_view))
+        apis = urlparser.get_apis(url_patterns)
+        serializers = generator._get_serializer_set(apis)
+        self.assertIn(CommentSerializer, serializers)
+        self.assertIn(QuerySerializer, serializers)
+        api = {
+            'path': 'a-path/',
+            'callback': func_to_wrapper(a_view),
+            'pattern': patterns('')
+        }
+        operations = generator.get_operations(api)
+        self.assertEqual(len(operations), 1)
+        parameters = operations[0]['parameters']
+        self.assertEqual(len(parameters), 1)
+        self.assertNotIn("pytype", parameters[0])
+        self.assertEqual(parameters[0]['name'], 'stuff')
+        self.assertEqual(parameters[0]['type'], 'QuerySerializer')
+
 
 class ViewMockerNeedingAPI(ListCreateAPIView):
     def get_serializer_class(self):
@@ -1705,3 +1876,100 @@ def my_view_mocker(view):
 
 def my_view_mocker2(view):
     pass
+
+
+reST_SETTINGS = {
+    'REST_FRAMEWORK': {
+        'VIEW_DESCRIPTION_FUNCTION':
+        'rest_framework_swagger.views.get_restructuredtext'
+    }
+}
+
+
+@override_settings(**reST_SETTINGS)
+class RESTDocstringTests(TestCase):
+    def test_get_summary_empty(self):
+        class MyViewSet(ModelViewSet):
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = make_viewset_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("", summary)
+
+    def test_get_summary_view(self):
+        class MyViewSet(ModelViewSet):
+            """
+            Oh yes this is reST
+            ============
+            """
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = make_viewset_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        summary = introspector.get_summary()
+        self.assertEqual("Oh yes this is reST", summary)
+
+    def test_get_yaml(self):
+        class MyViewSet(ModelViewSet):
+            """
+            Oh yes this is reST
+            ---
+            # Oh no, this isn't reST
+            create:
+                param: my param
+            """
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = make_viewset_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        docs = introspector.get_notes()
+        self.assertIn('Oh yes this is reST', docs)
+        doc_parser = introspector.get_yaml_parser()
+        self.assertEqual(doc_parser.object['param'], 'my param')
+
+    def test_dont_get_yaml(self):
+        class MyViewSet(ModelViewSet):
+            """
+            Oh yes this is reST
+            -------------------
+            Oh yes so is this
+            """
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = make_viewset_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        docs = introspector.get_notes()
+        self.assertIn('Oh yes this is reST', docs)
+        self.assertIn('Oh yes so is this', docs)
+
+    def test_dont_get_params(self):
+        class MyViewSet(ModelViewSet):
+            """
+            Oh yes this is reST
+            -------------------
+            Oh yes so is this
+            -- this isnt
+            """
+            model = User
+            serializer_class = CommentSerializer
+            paginate_by = 20
+            paginate_by_param = 'page_this_by'
+
+        class_introspector = make_viewset_introspector(MyViewSet)
+        introspector = get_introspectors(class_introspector)['create']
+        docs = introspector.get_notes()
+        self.assertIn('Oh yes this is reST', docs)
+        self.assertIn('Oh yes so is this', docs)
